-- Joust v5
-- Based on the Williams arcade game of the same name (1982)
-- by irons (jonirons@gmail.com)

-- Flap/fly using the action key. Move forward with trigger 1, back
-- with trigger 2. Collide with other players; the player with greater
-- altitude kills the other player. Large, open maps
-- recommended. Compatible with any scenario. Gather games as Every
-- Man For Himself (score is based on kills, not points). However,
-- variants such as playing King of the Hill or other built-in
-- gametypes with this script might be interesting. Players of the
-- same team are all the same color; friendly fire is always on.

Triggers = {}

local attackDistance = 1 -- distance in WU for fatal collision
local ceilingBounceDistance = .25 -- WU before ceiling bounces the player
local killerProjectile = "compiler bolt minor"
local flapUp = 0.019
local flapMove = 0.005
local flapSound = "enforcer attack"
local fartCloudEffect = "rocket contrail"
local invincibilityTime = 300
local respawnSound = "got ball"
local ceilingBounceSound = "ball bounce"

-- Extra gametype stuff
local ballPenalty = .75 -- flight accel is cut when carrying a ball

function Triggers.init()
   local teams = testTeams()
   for p in Players() do
      if teams and p.color ~= p.team then p.color = p.team end
   end
end

function Triggers.idle()
   for p1 in Players() do
      if Game.ticks == 0 then
	 p1._noob = true
	 revived(p1)
      end
      help(p1)
      disable(p1)
      checkInvincibility(p1)
      checkBounce(p1)
      if not p1.dead then
	 flap(p1)
	 for p2 in Players() do
	    calculateCollision(p1, p2)
	 end
      end
   end
end

function Triggers.player_killed(victim, aggressor, action, projectile)
   aggressor._noob = false
end

function Triggers.player_revived(player)
   revived(player)
end

function revived(player)
   player.invincibility_duration = invincibilityTime
   player.monster:play_sound(respawnSound)
end

function checkBounce(player)
   if player.monster.polygon.ceiling.z -
      (player.monster.z +
       player.monster.type.height) <= ceilingBounceDistance
   and player.monster.z >
   player.monster.polygon.floor.z + ceilingBounceDistance
   then
      bounce(player)
   end
end

function bounce(player)
   player:accelerate(player.yaw, 0, -3 * flapUp)
   player.monster:play_sound(ceilingBounceSound)
end

function disable(player)
   player.weapons.active = false
end

function checkInvincibility(player)
   if player.external_velocity.x ~=0 or
      player.external_velocity.y ~=0 or
      player.external_velocity.z ~= 0 then
      player.invincibility_duration = 0
   end
end

function flap(player)
   local accelModifier = 1
   if player.items["ball"] > 0 then
      accelModifier = ballPenalty
   end
   if player.action_flags.action_trigger then
      player.action_flags.action_trigger = false
      player:accelerate(player.yaw, 0, (accelModifier + .1) * flapUp)
      player.monster:play_sound(flapSound)
      fartCloud(player)
   end
   if player.action_flags.left_trigger then
      -- move player forward
      player.action_flags.left_trigger = false
      player:accelerate(player.yaw, accelModifier * flapMove, 0)
   end
   if player.action_flags.right_trigger then
      -- move player back
      player.action_flags.right_trigger = false
      player:accelerate(player.yaw, -1 * accelModifier *  flapMove, 0)
   end
end

function fartCloud(player)
   local fart = Effects.new(player.monster.x, player.monster.y,
			    player.monster.z + player.monster.type.height/2,
			    player.monster.polygon,
			    fartCloudEffect)
end

function calculateCollision(p1, p2)
   if not p2.dead and p1 ~= p2 then
      if dist(p1, p2) <= attackDistance then
	 if p1.monster.z > p2.monster.z then
	    kill(p2, p1)
	 elseif p2.monster.z > p1.monster.z then
	    kill(p1, p2)
	 end
      end
   end
end

function kill(victim, killer)
   if victim.invincibility_duration <= 0 then
      victim.life = 0
      local projectile =
	 Projectiles.new(victim.monster.x, victim.monster.y,
			 victim.monster.z + victim.monster.type.height,
			 victim.monster.polygon,
			 killerProjectile)
      projectile.yaw = victim.yaw
      projectile.pitch = -90
      projectile.owner = killer.monster  
   end
end

function dist(p1, p2)
   return math.sqrt(
      (p1.monster.x - p2.monster.x)^2 +
      (p1.monster.y - p2.monster.y)^2 +
      (p1.monster.z - p2.monster.z)^2)
end

function help(player)
   if player._noob then
      player.overlays[0].text = "New to Joust?"
      player.overlays[1].text = "Flap: action"
      player.overlays[2].text = "Forward/Back: hold triggers"
      player.overlays[3].text = " "
      player.overlays[4].text = "In a collision, the higher player wins."
      for i=0,5 do
	 player.overlays[i].color = "green"
      end
   else
      for i=0,5 do
	 player.overlays[i]:clear()
      end
   end
end

function testTeams()
   local colors = {}
   for c in PlayerColors() do
      colors[tostring(c)] = 0
   end
   for p in Players() do
      colors[tostring(player.team)] = colors[tostring(player.team)] + 1
   end
   for c in PlayerColors() do
      if colors[tostring(c)] > 1 then return true end
   end
   return false
end
